BufferedInput

Struct BufferedInput 

pub struct BufferedInput<R> {
    reader: R,
    buffer: Vec<u8>,
    input: FastInput,
    filled: usize,
    eof: bool,
}
Buffered token reader with the same token requirements as FastInput. Tokens may span reads and borrowed tokens remain valid until the next scan.

Fields

reader: Rbuffer: Vec<u8>input: FastInputfilled: usizeeof: bool

Implementations

Source

impl<R: Read> BufferedInput<R>

pub unsafe fn new(reader: R) -> Self

Safety

Every scan must satisfy FastInput’s token requirements, including UTF-8.

fn refill(&mut self) -> bool

Examples found in repository?
crates/competitive/src/tools/fastio.rs (line 741)
738    fn skip_whitespace(&mut self) {
739        loop {
740            self.input.skip_whitespace();
741            if self.input.ptr < self.input.end || !self.refill() {
742                break;
743            }
744        }
745    }
746
747    #[inline]
748    fn next_token(&mut self) -> Option<&str> {
749        if self.input.ptr >= self.input.end && !self.refill() {
750            return None;
751        }
752        Some(unsafe { std::str::from_utf8_unchecked(self.input.bytes()) })
753    }